import java.util.LinkedList;
import java.util.List;
import java.util.Queue;

@SuppressWarnings("unchecked")
public class SuspiciousBackpack {

	int numSuspects;
	List<Integer>[] graph;
	int[] inDegree;

	SuspiciousBackpack(int numSuspects){
		this.numSuspects = numSuspects;
		graph = new List[numSuspects * 2]; //graph[i] -> i enters; graph[numSuspects + i] -> i leaves.
		inDegree = new int[numSuspects * 2];

		//Quando entram
		for(int i = 0; i < numSuspects; i++){
			graph[i] = new LinkedList();
			graph[i].add(numSuspects+i); //adiciona saida como sucessor
			inDegree[numSuspects+i]++;
		}

		//Quando saem
		for(int i = numSuspects; i < numSuspects*2; i++){
			graph[i] = new LinkedList();
		}
	}

	void addPrecedence(int s1, int s2){
		graph[numSuspects + s1].add(s2);
		inDegree[s2]++;
	}

	void addConcurrence(int s1, int s2){
		graph[s1].add(numSuspects + s2);
		inDegree[numSuspects + s2]++;

		graph[s2].add(numSuspects + s1);
		inDegree[numSuspects + s1]++;
	}

	boolean isConsistent(){
		int numProc = 0;
		Queue<Integer> ready = new LinkedList();

		for(int i = 0; i< inDegree.length; i++){
			if(inDegree[i] == 0) {
				ready.add(i);
			}
		}

		while(!ready.isEmpty()){
			int currNode = ready.remove();
			numProc++;

			for(int node: graph[currNode]){
				if(--inDegree[node] == 0) {
					ready.add(node);
				}
			}
		}

		return numProc == graph.length;
	}
}
