import java.util.LinkedList;
import java.util.Queue;

@SuppressWarnings("unchecked")
public class JackOLantern {

	private int width, height;
	private int[][] grid, mapLanterns;

	public JackOLantern(int width, int height){
		this.width = width;
		this.height = height;
		grid = new int[height][width]; //[linha][coluna]
		mapLanterns = new int[height][width];
	}

	private class State{
		protected int lantern;
		protected int x, y;

		State(int lantern, int x, int y){
			this.lantern = lantern;
			this.x = x;
			this.y = y;
		}
	}

	public void addLine(String[] lineElement, int line){
		for(int col = 0; col < width; col++)
			if (lineElement[col].equals("*"))
				grid[line][col] = -1;
			else
				grid[line][col] = Integer.parseInt(lineElement[col]);
	}

	public int solve(){
		Queue<State> ready = new LinkedList();
		int result = 0, counter;

		if(grid[0][0] == -1){
			ready.add(new State(0, 1, 0));
			ready.add(new State(0, 0, 1));
		} else if(grid[0][0] != 0){
			ready.add(new State(grid[0][0] - 1 , 1, 0));
			ready.add(new State(grid[0][0] - 1, 0, 1));
		} else {
			if(grid[0][1] == -1)
				ready.add(new State(0, 1, 0));

			if(grid[1][0] == -1)
				ready.add(new State(0, 0, 1));
		}

		if(ready.isEmpty()) {
			return -1;
		}
		result++;
		counter = ready.size();

		int x, y;
		do {
			if(ready.isEmpty())
				return -1;
			State currState = ready.remove();

			x = currState.x;
			y = currState.y;

			if(grid[y][x] > currState.lantern) { //Se houver uma lanterna na posicao atual
				currState.lantern = grid[y][x];
				grid[y][x] = 0; //Garantee that it doesnt use the lantern again
			}

			if(mapLanterns[y][x] == 0)
				mapLanterns[y][x] = -1;
			else
				if (currState.lantern > mapLanterns[y][x])
					mapLanterns[y][x] = currState.lantern;

			if(grid[y][x] == -1) {
				//Se ja tiver visitado mas tiver lanterna pode valer a pena revisitar, caso contrario nao
				if (x < height - 1 && (mapLanterns[y][x + 1] == 0 || currState.lantern > mapLanterns[y][x + 1]))
					ready.add(new State(currState.lantern, x + 1, y));

				if (y < height - 1 && (mapLanterns[y + 1][x] == 0 || currState.lantern > mapLanterns[y + 1][x]))
					ready.add(new State(currState.lantern, x, y + 1));

				if (x > 0 && (mapLanterns[y][x -1] == 0 || currState.lantern > mapLanterns[y][x - 1]))
					ready.add(new State(currState.lantern, x - 1, y));

				if (y > 0 && (mapLanterns[y - 1][x] == 0 || currState.lantern > mapLanterns[y - 1][x]))
					ready.add(new State(currState.lantern, x, y - 1));

			} else if(currState.lantern > 0){ //Se existe uma lampada na posicao
				//Nunca se verifica se ja foi visitado
				if(x < height - 1 && (mapLanterns[y][x + 1] == 0 || currState.lantern > mapLanterns[y][x + 1]))
					if (grid[y][x + 1] == -1)
						ready.add(new State(currState.lantern, x + 1, y));
					else
						ready.add(new State(currState.lantern - 1, x + 1, y));

				if(y < height - 1 && (mapLanterns[y + 1][x] == 0 || currState.lantern > mapLanterns[y + 1][x]))
					if (grid[y + 1][x] == -1)
						ready.add(new State(currState.lantern, x, y + 1));
					else
						ready.add(new State(currState.lantern - 1, x, y + 1));

				if(x > 0 && (mapLanterns[y][x - 1] == 0 || currState.lantern > mapLanterns[y][x - 1]))
					if (grid[y][x - 1] == -1)
						ready.add(new State(currState.lantern, x - 1, y));
					else
						ready.add(new State(currState.lantern - 1, x - 1, y));

				if(y > 0 && (mapLanterns[y - 1][x] == 0 || currState.lantern > mapLanterns[y - 1][x]))
					if (grid[y - 1][x] == -1)
						ready.add(new State(currState.lantern, x, y - 1));
					else
						ready.add(new State(currState.lantern - 1, x, y - 1));

			} else {
				//Quando nao estamos num sitio iluminado nem temos lanterna
				if(x < width - 1 && grid[y][x+1] == -1 && mapLanterns[y][x + 1] == 0)
					ready.add(new State(currState.lantern, x+1, y));

				if(y < height - 1 && grid[y+1][x] == -1 && mapLanterns[y+1][x] == 0)
					ready.add(new State(currState.lantern, x, y+1));

				if(x > 0 && grid[y][x-1] == -1 && mapLanterns[y][x-1] == 0)
					ready.add(new State(currState.lantern, x-1, y));

				if(y > 0 && grid[y-1][x] == -1 && mapLanterns[y-1][x] == 0)
					ready.add(new State(currState.lantern, x, y-1));
			}

			if(--counter == 0){
				result++;
				counter = ready.size();
			}

		} while(x < width - 1 || y < height - 1);

		return result;
	}
}